@@ -13,10 +13,18 @@ import datetime
 class WeChatData:
     errcode = 0
     errmsg = "ok"
+    request_result = None
+    result_json = None
 
     def __str__(self):
         return str(self.__dict__)
 
+    def set_request_result(self, request_result):
+        self.request_result = request_result
+
+    def set_result_json(self, result_json):
+        self.result_json = result_json
+
 
 class LogsConf:
 
@@ -135,7 +143,8 @@ class WeRequest:
 
         return result
 
-    def post(self, url, data=None, encoding="utf-8", ensure_ascii=False, content_type="json", **kwargs):
+    def post(self, url, data=None, encoding="utf-8", ensure_ascii=False, content_type="json",
+             headers={'content-type': 'application/json'}, **kwargs):
         """
         get请求
         :param url: 路径
@@ -151,28 +160,36 @@ class WeRequest:
                     WeChatData.request_result: <requests.Response>
                     WeChatData... 微信返回的对象
         """
-
         if isinstance(data, dict):
             data = json.dumps(data, ensure_ascii=ensure_ascii)
 
-        if encoding:
+        if data and encoding:
             # 有些接口需要编码之后才生效
             data = data.encode(encoding)
 
         logger = self.logger
-        result = requests.post(url, data=data, headers={'content-type': 'application/json'}, **kwargs)
+        result = requests.post(url, data=data, headers=headers, **kwargs)
         logger.debug("接口请求参数 >>> 微信api")
-        logger.debug("路径：%s" % result.url)
+        logger.debug("路径：%s" % result.url.split('?')[0])
         logger.debug("请求方式：POST")
+        logger.debug("请求参数(params)：%s" % kwargs.get("params"))
         logger.debug("请求体(data)：%s" % data)
         logger.debug("——————————————————————————————————————————————————————————")
         logger.debug("")
-
+        print("类型:", result.headers)
         if content_type == "json":
-            obj = self.deal_with(result)
-            return obj
+            return self.deal_with(result)
         elif content_type == "media":
             return result.content
+        elif content_type == "auto":
+            _content_type = result.headers.get("Content-Type")
+            if _content_type in [None, "image/gif", "image/jpeg", "application/pdf", "application/msword",
+                                 "application/octet-stream"]:
+                return result.content
+            elif _content_type in ['text/plain', 'application/json']:
+                return self.deal_with(result)
+            else:
+                return result
 
         return result
 
@@ -204,13 +221,19 @@ class WeRequest:
             # access_token 错误
             raise WeAccessTokenExpired()
 
+        # 空列表不报错
+        if data_json["errcode"] == 9410000:
+            data_json["errcode"] = 0
+
         if data_json["errcode"] != 0:
             raise WeError(result)
 
-        data_json['request_result'] = result
-        data_json['result_json'] = result.json()
+        # data_json['request_result'] = result
+        # data_json['result_json'] = result.json()
 
         wechat_data = dict2obj(data_json)
+        wechat_data.set_request_result(result)
+        wechat_data.set_result_json(data_json)
 
         return wechat_data
 
@@ -227,6 +250,7 @@ class WeChatBase:
     access_token = None
     cache = None
     passive_access_token = True  # 被动刷新access_token
+    get_access_token_url = None  # 获取access_token的url
 
     def __init__(self, appid, secret, debug=False, access_token=None, passive_access_token=True, **kwargs):
         """
@@ -241,11 +265,19 @@ class WeChatBase:
                                     True: 当access_token 失效之后，自动获取新的access_token，并缓存
                                     False: 当access_token 失效之后抛出异常，需要主动调用get_access_token_cache() 接口刷新
         :param kwargs:
+                get_access_token_url: 设置获取接口凭证的连接（自定义的接口），通过一个统一的中间接口获取接口凭证，避免接口凭证冲突
+                                    用法：
+                                        注：这个用法一般在非生产环境（多环境）下使用，接口一般为生产环境接口，保证这个接口的稳定性
+                                        1、写一个接口，这个接口的功能是获取access_token并缓存，然后返回access_token
+                                        2、配置get_access_token_url为第一步的接口url，如：https://127.0.0.1:8000/api/get_access_token
+                                        3、开启自动更新access_token即可，工具会自动从第二步的地址中获取access_token
+
         """
         self.appid = appid
         self.secret = secret
         self.DEBUG = debug
         self.passive_access_token = passive_access_token
+        self.get_access_token_url = kwargs.get("get_access_token_url")
 
         # 初始化日志
         self.logs_conf = self.get_logs_conf()
@@ -304,10 +336,44 @@ class WeChatBase:
         except WeAccessTokenExpired as e:
             # 更新传参 或抛出异常
             par = self.handle_access_token_expired(e, par)
-            result = self.wx_request.post(**par, **kwargs)  # 从新发起请求
+            result = self.wx_request.post(**par, **kwargs)  # 重新发起请求
 
         return result
 
+    def file_post(self, url, file_name, file, params=None, **kwargs):
+        """
+        上传media文件
+        :param url: url
+        :param file_name: 文件名
+        :param file: 二进制文件
+        :param params: 请求参数
+        :param kwargs:
+        :return: 返回request请求结果
+        """
+        par = get_parameter(locals())
+        self.suffix_check(file_name)  # 检查名称后缀
+
+        try:
+            result = self.wx_request.post(url, params=params, files={"media": (file_name, file)}, encoding="",
+                                          headers={'content-type': 'multipart/form-data'})
+        except WeAccessTokenExpired as e:
+            # 更新传参 或抛出异常
+            par = self.handle_access_token_expired(e, par)
+            result = self.wx_request.post(**par, **kwargs)  # 重新发起请求
+
+        return result
+
+    @staticmethod
+    def suffix_check(file_name):
+        """ 检查文件后缀（上传到微信素材管理用） """
+        suffix = file_name.split('.')
+        suffix = suffix[1] if len(suffix) >= 2 else None
+        if not suffix:
+            raise ValueError("缺少文件后缀")
+        ava_suffix = ["PNG", "JPEG", "JPG", "GIF", "AMR", "MP3", "MP4"]
+        if suffix.upper() not in ava_suffix:
+            raise ValueError("不支持的文件后缀（%s），仅支持以下格式：%s" % (suffix, "、".join(ava_suffix)))
+
     def get_access_token(self):
         """
         获取access_token
@@ -318,14 +384,17 @@ class WeChatBase:
         官方文档：
         https://developers.weixin.qq.com/miniprogram/dev/api-backend/open-api/access-token/auth.getAccessToken.html
 
-        :return: 返回WeChatDate对象对象包含参数：
+        :return: 返回WechatData对象对象包含参数：
                 属性                  类型              说明
                 errcode             number          错误码，正常时返回0
                 errmsg              str             错误描述，正常时返回ok
                 access_token        string          获取到的调用接口凭证
-                expires_in          string          有效时长
+                expires_in          string          有效时长，可能为空
 
         """
+        if self.get_access_token_url:
+            return self.post(self.get_access_token_url, data={"appid": self.appid, "secret": self.secret})
+
         url = "https://api.weixin.qq.com/cgi-bin/token"
         params = self.get_full_params(grant_type="client_credential")
         return self.get(url, params=params)
